<?php
/**
 *
 *   Hepsipay adına Alttantire Yazılım Çözümleri tarafından geliştirilmiştir.
 *
 * @author      Alttantire <info@alttantire.com>
 * @date        2025
 *
 */

class HepsipayPaymentModuleFrontController extends ModuleFrontController {
	/**
	 * This class should be use by your Instant Payment
	 * Notification system to validate the order remotely
	 */

	public function initContent() {
		$this->display_column_left = false;
		$this->display_column_right = false;
		$this->bootstrap = true;

		parent::initContent();

		/**
		 * If the module is not active anymore, no need to process anything.
		 */

		if ( ! $this->module->active ) {
			die;
		}

		if ( Tools::getValue( 'spinner' ) ) {
			$this->display_header = false;
			$this->display_footer = false;
			$this->display_column_left = false;
			$this->display_column_right = false;

			return $this->setTemplate( 'module:' . $this->module->name . '/views/templates/front/spinner.tpl' );
		}

		$cart = Context::getContext()->cart;
		$currency = Context::getContext()->currency;
		$customer = Context::getContext()->customer;

		$this->addJquery();

		if ( ! $cart->id ) {
			Tools::redirectLink( __PS_BASE_URI__ );
			exit;
		}

		if ( ! isset( $this->module->gateway ) || ! $this->module->verified ) {
			$this->errors[] = $this->module->l( 'Ödeme yöntemi aktif değil. Lütfen site yöneticisi ile iletişime geçin.' );

			return $this->setTemplate( 'module:' . $this->module->name . '/views/templates/front/error.tpl' );

		}

		if ( $currency->id != Configuration::get( 'HEPSIPAY_TL_CURRENCY' ) ) {
			$this->context->smarty->assign(
				array(
					'data'       => array(
						"total"         => $cart->getOrderTotal( true, 3 ),
						"verify_client" => false,
						'frame_url'     => "",
						'callback_url'  => "",
						'message'       => $this->module->l( 'Hepsipay ile sadece Türk Lirası ile ödeme yapabilirsiniz.' ),
					),
					'module_dir' => _MODULE_DIR_ . $this->module->name,
				)
			);

			return $this->setTemplate( 'module:' . $this->module->name . '/views/templates/front/payform.tpl' );

		}

		if ( Tools::getValue( 'callback' ) ) {

			$processPayment = $this->processPayment();

			if ( $processPayment["status"] ) {

				$redirect_url = Context::getContext()->link->getPageLink(
					'order-confirmation',
					true,
					null,
					'id_cart=' . (int) $cart->id .
					'&id_module=' . (int) $this->module->id .
					'&id_order=' . (int) $this->module->currentOrder .
					'&key=' . $customer->secure_key
				);

				if ( version_compare( _PS_VERSION_, '1.7.0', '>=' ) === true ) {
					$redirect_url = Context::getContext()->link->getPageLink(
						'order-confirmation',
						true,
						null,
						array(
							'id_cart'   => $cart->id,
							'id_module' => $this->module->id,
							'id_order'  => $this->module->currentOrder,
							'key'       => $customer->secure_key
						)
					);

				}

				die( json_encode( array(
					'redirect_url' => $redirect_url,
				) ) );

			} else {

				die( json_encode( array(
					'redirect_url' => $this->context->link->getModuleLink(
						$this->module->name,
						'payment',
						array( "m" => $processPayment["message"] ),
						Configuration::get( 'PS_SSL_ENABLED' )
					),
				) ) );
			}

		}

		$frame_response = $this->getFrameUrl();

		$data = array(
			"total"         => $cart->getOrderTotal( true, 3 ),
			"verify_client" => $this->module->verified && $frame_response["verified"],
			'frame_url'     => $frame_response["frame_url"],
			'callback_url'  => $this->context->link->getModuleLink(
				$this->module->name,
				'payment',
				array( 'callback' => 1 ),
				Configuration::get( 'PS_SSL_ENABLED' )
			),
			'message'       => $frame_response["text_description"] ?
				$frame_response["text_description"] : Tools::getValue( 'm' ),
		);

		$this->context->smarty->assign( array(
			'data'       => $data,
			'module_dir' => _MODULE_DIR_ . $this->module->name,
		) );

		return $this->setTemplate( 'module:' . $this->module->name . '/views/templates/front/payform.tpl' );

	}

	public function processPayment() {

		$cart = Context::getContext()->cart;
		$customer = Context::getContext()->customer;
		$currency = Context::getContext()->currency;

		$paymentStatus = $this->checkPayment();

		if ( $paymentStatus["status"] ) {

			$payment_response = $paymentStatus["payment_response"];

			$this->module->validateOrder(
				$cart->id, Configuration::get( 'PS_OS_PAYMENT' ),
				$cart->getOrderTotal( true, 3 ),
				$this->module->displayName,
				null,
				null,
				$cart->id_currency,
				false,
				$customer->secure_key
			);

			$amount = ( $payment_response->Amount / 100 );
			$order = new Order( $this->module->currentOrder );
			$payment_col = $order->getOrderPayments();
			$payment = new OrderPayment( $payment_col[0]->id );
			$payment->transaction_id = $payment_response->MerchantOrderNumber;
			$payment->card_number = $payment_response->PaymentTypeDescription;
			$payment->amount = $amount;
			$payment->save();

			$notes[] = "Sepet Tutarı: " . $cart->getOrderTotal( true, 3 ) . $currency->iso_code;
			$notes[] = "Taksit: " . ( $payment_response->InstallmentCount > 0 ? $payment_response->InstallmentCount : "Yok" );
			$notes[] = "Vade Farkı: " . ( $payment_response->InterestAmount ) . $currency->iso_code;
			$notes[] = "Ödenen Tutar: " . ( $amount ) . $currency->iso_code;

			$orderMessage = new Message();
			$orderMessage->id_order = $order->id;
			$orderMessage->message = implode( "\n", $notes );
			$orderMessage->save();

			return [
				"status"  => true,
				"message" => null,
			];

		} else {

			return [
				"status"  => false,
				"message" => $paymentStatus["message"],
			];
		}

	}

	private function checkPayment() {
		$hepsipay_payment_data = json_decode(
			filter_input( INPUT_POST, "hepsipay_payment_data", FILTER_DEFAULT )
		);

		if ( ! is_object( $hepsipay_payment_data ) || ! isset( $hepsipay_payment_data->token ) ) {
			return null;
		}

		$token = $hepsipay_payment_data->token;
		$payment_response = $this->module->gateway->payment_status( $token );

		if ( ! is_object( $payment_response ) ) {
			$response = [
				"status"  => false,
				"message" => "Hatalı işlem..",
			];
		} elseif ( in_array( $payment_response->Status, [ 4, 5, 12, 14 ] ) ) {

			$response = [
				"status"           => true,
				"message"          => null,
				"payment_response" => $payment_response,
			];

		} else {
			$response = [
				"status"  => false,
				"message" => $payment_response->StatusDescription,
			];
		}

		return $response;
	}

	private function getFrameUrl() {

		$data = [ "verified" => false, "frame_url" => "", "text_description" => "" ];

		$cart = Context::getContext()->cart;

		$this->module->gateway->set_account_key(
			(string) $this->getCustomerMobilePhoneNumber()
		);

		$this->module->gateway->set_amount( $cart->getOrderTotal() * 100 );

		$this->module->gateway->set_merchant_callback_url(
			$this->context->link->getModuleLink(
				$this->module->name,
				'payment',
				array( 'spinner' => 1 ),
				Configuration::get( 'PS_SSL_ENABLED' )
			)
		);

		$this->module->gateway->set_merchant_order_number(
			$cart->id
		);
		$this->module->gateway->set_is_guest_user( true );
		$this->module->gateway->set_force_3ds( true );
		$this->module->gateway->set_allowed_installments( [
			1,
			2,
			3,
			4,
			5,
			6,
			7,
			8,
			9,
			10,
			11,
			12,
		] );

		$this->module->gateway->set_frame_type( 1 );

		$this->module->gateway->set_basket_item( [
			"Product"                => "Sipariş",
			"Price"                  => $cart->getOrderTotal() * 100,
			"SubMerchantMemberId"    => null,
			"SubMerchantMemberPrice" => null,
			"CategoryId"             => null,
			"ExternalId"             => $cart->id,
		] );

		try {
			// Check Availability
			$frame_check_availability_response = $this->module->gateway->check_availability();

			if ( $frame_check_availability_response->Success === true ) {
				$frame_init_response = $this->module->gateway->frame_init();

				if ( $frame_init_response && $frame_init_response->Success === true ) {
					$frame_url = $frame_init_response->FrameUrl;
					$data["verified"] = true;
					$data["frame_url"] = $frame_url;
				} else {
					$data["text_description"] =
						$frame_init_response->Message;
				}

			} else {
				$data["text_description"] = $frame_check_availability_response->Message;
			}
		} catch ( InvalidArgumentException $e ) {
			$data["text_description"] = (string) $e->getMessage();
		}

		return $data;
	}

	private function getCustomerMobilePhoneNumber() {

		$cart = Context::getContext()->cart;

		$address_obj = new Address( $cart->id_address_invoice );
		$phone = $address_obj->phone_mobile ? $address_obj->phone_mobile : $address_obj->phone;

		$customer_phone = (int) preg_replace(
			"/\D/",
			"",
			$phone
		);

		return substr( $customer_phone, 0, 2 ) !== "90" ? "90" . $customer_phone : $customer_phone;

	}

	protected function isValidOrder() {
		/**
		 * Add your checks right there
		 */
		return true;
	}

}
