<?php
/**
 *
 *   Hepsipay adına Alttantire Yazılım Çözümleri tarafından geliştirilmiştir.
 *
 * @author      Alttantire <info@alttantire.com>
 * @date        2025
 *
 */

namespace Alttantire;

use Exception;

class HepsipayGateway
{


	/**
	 * @var string
	 */
	protected $url;
	/**
	 * @var string
	 */
	protected $endpoint;
	/**
	 * @var string
	 */
	protected $merchant_no;
	/**
	 * @var string
	 */
	protected $terminal_no;
	/**
	 * @var string
	 */
	protected $store_key;
	/**
	 * @var string
	 */
	protected $communication_method = "cURL";
	/**
	 * @var int
	 */
	private $frame_type;
	/**
	 * @var int
	 */
	private $amount;
	/**
	 * @var string
	 */
	private $account_key;
	/**
	 * @var string
	 */
	private $merchant_callback_url;
	/**
	 * @var string
	 */
	private $merchant_order_number = "";
	/**
	 * @var string
	 */
	private $checkout_merchant_url = "";
	/**
	 * @var string
	 */
	private $bank_order_id = "";
	/**
	 * @var string
	 */
	private $external_id = "";
	/**
	 * @var string
	 */
	private $pos_alias;
	/**
	 * @var array
	 */
	private $basket_items;
	/**
	 * @var array
	 */
	private $allowed_installments = [];
	/**
	 * @var bool
	 */
	private $is_guest_user = false;
	/**
	 * @var bool
	 */
	private $force_3ds = false;
	/**
	 * @var int
	 */
	private $refund_amount;
	/**
	 * @var string
	 */
	private $client_ip;
	/**
	 * @var string
	 */
	private $payment_id;
	/**
	 * @var string
	 */
	private $reference_no;

	/**
	 * @param $endpoint
	 * @param $merchant_no
	 * @param $terminal_no
	 * @param $store_key
	 */
	public function __construct( $endpoint, $merchant_no, $terminal_no, $store_key ) {
		$this->endpoint = $endpoint;
		$this->merchant_no = $merchant_no;
		$this->terminal_no = $terminal_no;
		$this->store_key = $store_key;

		$this->init();
	}

	/**
	 * @return $this
	 */
	public function init() {
		if ( ! $this->ends_with( $this->endpoint, "/" ) ) {
			$this->endpoint = $this->endpoint . "/";
		}

		$this->detect_curl_availability();

		return $this;
	}

	private function ends_with( $haystack, $needle, $case = true ) {
		if ( $case ) {
			return strcmp( substr( $haystack, strlen( $haystack ) - strlen( $needle ) ), $needle ) === 0;
		}

		return strcasecmp( substr( $haystack, strlen( $haystack ) - strlen( $needle ) ), $needle ) === 0;
	}

	/**
	 * @param  $amount
	 *
	 * @return void
	 */
	public function set_amount( $amount ) {
		$this->amount = $amount;
	}

	/**
	 * @param   $account_key
	 *
	 * @return void
	 */
	public function set_account_key( $account_key ) {
		$this->account_key = $account_key;
	}

	/**
	 * @param   $merchant_callback_url
	 *
	 * @return void
	 */
	public function set_merchant_callback_url( $merchant_callback_url ) {
		$this->merchant_callback_url = $merchant_callback_url;
	}

	/**
	 * @param   $basket_item
	 *
	 * @return void
	 */
	public function set_basket_item( $basket_item ) {
		$this->basket_items[] = json_decode( json_encode( $basket_item ) );
	}

	/**
	 * @param   $is_guest_user
	 *
	 * @return void
	 */
	public function set_is_guest_user( $is_guest_user ) {
		$this->is_guest_user = $is_guest_user;
	}

	/**
	 * @param   $force_3ds
	 *
	 * @return void
	 */
	public function set_force_3ds( $force_3ds ) {
		$this->force_3ds = $force_3ds;
	}

	/**
	 * @param   $allowed_installments
	 *
	 * @return void
	 */
	public function set_allowed_installments( $allowed_installments ) {
		$this->allowed_installments = $allowed_installments;
	}

	/**
	 * @param   $merchant_order_number
	 *
	 * @return void
	 */
	public function set_merchant_order_number( $merchant_order_number ) {
		$this->merchant_order_number = $merchant_order_number;
	}

	/**
	 * @param   $bank_order_id
	 *
	 * @return void
	 */
	public function set_bank_order_id( $bank_order_id ) {
		$this->bank_order_id = $bank_order_id;
	}

	/**
	 * @param   $external_id
	 *
	 * @return void
	 */
	public function set_external_id( $external_id ) {
		$this->external_id = $external_id;
	}

	/**
	 * @param   $frame_type
	 *
	 * @return void
	 */
	public function set_frame_type( $frame_type ) {
		$this->frame_type = $frame_type;
	}

	/**
	 * @param   $checkout_merchant_url
	 *
	 * @return void
	 */
	public function set_checkout_merchant_url( $checkout_merchant_url ) {
		$this->checkout_merchant_url = $checkout_merchant_url;
	}

	/**
	 * @param   $pos_alias
	 *
	 * @return void
	 */
	public function set_pos_alias( $pos_alias ) {
		$this->pos_alias = $pos_alias;
	}

	/**
	 * @param   $refund_amount
	 *
	 * @return void
	 */
	public function set_refund_amount( $refund_amount ) {
		$this->refund_amount = $refund_amount;
	}

	/**
	 * @param   $client_ip
	 *
	 * @return void
	 */
	public function set_client_ip( $client_ip ) {
		$this->client_ip = $client_ip;
	}

	/**
	 * @param   $payment_id
	 *
	 * @return void
	 */
	public function set_payment_id( $payment_id ) {
		$this->payment_id = $payment_id;
	}

	/**
	 * @param   $reference_no
	 *
	 * @return void
	 */
	public function set_reference_no( $reference_no ) {
		$this->reference_no = $reference_no;
	}

	/**
	 * @return string
	 */
	public function get_request_body_string() {

		$body = [];

		$items = [
			"amount"                => "Amount",
			"account_key"           => "AccountKey",
			"merchant_callback_url" => "MerchantCallBackUrl",
			"basket_items"          => "BasketItems",
			"is_guest_user"         => "IsGuestUser",
			"force_3ds"             => "Force3ds",
			"allowed_installments"  => "AllowedInstallments",
			"frame_type"            => "FrameType",
			"merchant_order_number" => "MerchantOrderNumber",
			"bank_order_id"         => "BankOrderId",
			"external_id"           => "ExternalId",
			"pos_alias"             => "PosAlias",
			"refund_amount"         => "refundAmount",
			"client_ip"             => "clientIp",
			"payment_id"            => "paymentId",
			"reference_no"          => "referenceNo",
		];

		if ( isset( $this->frame_type ) && $this->frame_type === 2 ) {
			$body["CheckoutMerchantUrl"] = $this->checkout_merchant_url;
		}

		foreach ( $items as $key => $item ) {

			if ( ! empty( $this->{$key} ) ) {
				$body[ $item ] = $this->{$key};
			}

		}

		if ( empty( $body ) ) {
			return "";
		}

		return json_encode( $body );

	}

	/**
	 * @return void
	 */
	public function detect_curl_availability() {
		if ( function_exists( 'curl_init' ) ) {
			$this->communication_method = "cURL";

			return;
		}

		if ( function_exists( 'file_get_contents' ) && ini_get( 'allow_url_fopen' ) ) {
			$this->communication_method = "fopen";
		}
	}

	/**
	 * @throws Exception
	 */
	public function post( $url, $header_params = [] ) {
		if ( $this->communication_method === "cURL" ) {
			return $this->post_with_curl( $url, $header_params );
		} elseif ( $this->communication_method === "fopen" ) {
			return $this->post_with_file_contents( $url, $header_params );
		} else {
			return false;
		}
	}

	/**
	 * @throws Exception
	 */
	public function get( $url, $header_params = [] ) {
		if ( $this->communication_method === "cURL" ) {
			return $this->get_with_curl( $url, $header_params );
		} elseif ( $this->communication_method === "fopen" ) {
			return $this->get_with_file_contents( $url, $header_params );
		} else {
			return false;
		}
	}

	/**
	 * @throws Exception
	 */
	public function post_with_file_contents( $path, $header_params = [] ) {

		$path = $this->endpoint . $path;

		$header = [ "Content-Type: application/json" ];
		foreach ( $header_params as $key => $value ) {
			$header[] = $key . ": " . $value;
		}

		$data_string = ( $this->get_request_body_string() );

		$opts = array(
			'http' =>
				array(
					'method'  => 'POST',
					'header'  => implode( "\r\n", $header ),
					'timeout' => 10,
					'content' => $data_string
				)
		);

		$context = stream_context_create( $opts );

		try {

			$fp = @fopen( $path, 'r', false, $context );

			if ( ! $fp ) {
				throw new \InvalidArgumentException( $http_response_header[0] );
			}

			$response = stream_get_contents( $fp );

			$meta_data = stream_get_meta_data( $fp );
			$response_headers = $meta_data['wrapper_data'];

			fclose( $fp );

			$http_code = null;
			foreach ( $response_headers as $header ) {
				if ( preg_match( '#HTTP/\d+\.\d+ (\d+)#', $header, $matches ) ) {
					$http_code = intval( $matches[1] );
					break;
				}
			}

			if ( $http_code === 401 ) {
				throw new \InvalidArgumentException( "Yetkisiz Erişim. " . $http_code );
			}

			return json_decode( (string) $response );

		} catch ( Exception $e ) {
			throw new \InvalidArgumentException( "İstisna yakalandı: " . $e->getMessage() );
		}

	}

	/**
	 * @throws Exception
	 */
	public function get_with_file_contents( $path, $header_params = [] ) {

		$header = [ "Content-Type: application/json" ];
		foreach ( $header_params as $key => $value ) {
			$header[] = $key . ": " . $value;
		}

		try {

			$url = $this->endpoint . $path;

			$options = array(
				'http' => array(
					'header' => implode( "\r\n", $header ),
					'method' => 'GET',
				)
			);

			$context = stream_context_create( $options );

			$response = file_get_contents( $url, false, $context );

			if ( $response === false ) {
				throw new \InvalidArgumentException( "Hata oluştu " );
			}

			return json_decode( (string) $response );

		} catch ( Exception $e ) {
			throw new \InvalidArgumentException( "İstisna yakalandı: " . $e->getMessage() );
		}

	}

	/**
	 * @throws Exception
	 */
	public function post_with_curl( $url, $header_params = [] ) {
		try {

			$header = [ "Content-Type: application/json" ];
			foreach ( $header_params as $key => $value ) {
				$header[] = $key . ": " . $value;
			}

			$data_string = ( $this->get_request_body_string() );

			$ch = @curl_init( $this->endpoint . $url );

			@curl_setopt( $ch, CURLOPT_POST, true );
			@curl_setopt( $ch, CURLOPT_POSTFIELDS, $data_string );
			@curl_setopt( $ch, CURLOPT_HTTPHEADER, $header );
			@curl_setopt( $ch, CURLOPT_RETURNTRANSFER, true );
			@curl_setopt( $ch, CURLOPT_CONNECTTIMEOUT, 5 );
			@curl_setopt( $ch, CURLOPT_TIMEOUT, 10 ); //timeout in seconds

			$result = curl_exec( $ch );

			if ( @curl_errno( $ch ) ) {
				$error_msg = curl_error( $ch );
				@curl_close( $ch );
				throw new \InvalidArgumentException( $error_msg );
			} else {

				$http_code = curl_getinfo( $ch, CURLINFO_HTTP_CODE );
				$header_size = curl_getinfo( $ch, CURLINFO_HEADER_SIZE );
				$header = substr( $result, 0, $header_size );

				if ( $http_code === 401 ) {
					throw new \InvalidArgumentException( "Yetkisiz Erişim. " . $http_code );
				}

				$data = json_decode( $result );
				@curl_close( $ch );

				return $data;

			}

		} catch ( Exception $e ) {

			throw new \InvalidArgumentException( $e->getMessage() );
		}

	}

	/**
	 * @throws Exception
	 */
	public function get_with_curl( $url, $header_params = [] ) {

		try {

			$header = [ "Content-Type: application/json" ];
			foreach ( $header_params as $key => $value ) {
				$header[] = $key . ": " . $value;
			}

			$ch = @curl_init( $this->endpoint . $url );

			@curl_setopt( $ch, CURLOPT_HTTPHEADER, $header );
			@curl_setopt( $ch, CURLOPT_RETURNTRANSFER, true );
			@curl_setopt( $ch, CURLOPT_CONNECTTIMEOUT, 5 );
			@curl_setopt( $ch, CURLOPT_TIMEOUT, 10 ); //timeout in seconds

			$result = curl_exec( $ch );

			if ( @curl_errno( $ch ) ) {
				$error_msg = curl_error( $ch );
				@curl_close( $ch );
				throw new \InvalidArgumentException( $error_msg );
			} else {

				$http_code = curl_getinfo( $ch, CURLINFO_HTTP_CODE );
				$header_size = curl_getinfo( $ch, CURLINFO_HEADER_SIZE );
				$header = substr( $result, 0, $header_size );

				if ( $http_code === 401 ) {
					throw new \InvalidArgumentException( "Yetkisiz Erişim. " . $http_code );
				}

				$data = json_decode( $result );
				@curl_close( $ch );

				return $data;

			}

		} catch ( Exception $e ) {

			throw new \InvalidArgumentException( $e->getMessage() );
		}

	}

	public function get_guid() {
		if ( function_exists( 'com_create_guid' ) === true ) {
			return trim( com_create_guid(), '{}' );
		}

		return sprintf( '%04X%04X-%04X-%04X-%04X-%04X%04X%04X',
			mt_rand( 0, 65535 ),
			mt_rand( 0, 65535 ),
			mt_rand( 0, 65535 ),
			mt_rand( 16384, 20479 ),
			mt_rand( 32768, 49151 ),
			mt_rand( 0, 65535 ),
			mt_rand( 0, 65535 ),
			mt_rand( 0, 65535 ) );
	}

	/**
	 * @param   $params
	 *
	 * @return array
	 */
	public function params( $params = [] ) {
		return [
			       'merchant-no' => $this->merchant_no,
			       'terminal-no' => $this->terminal_no,
		       ] + $params;
	}

	/**
	 * Signature üretme methodu.
	 *
	 * @param $url string
	 *
	 * @return string
	 */
	private function generate_signature( $url ) {
		$hashString = $this->endpoint .
		              $url .
		              $this->merchant_no .
		              $this->terminal_no .
		              $this->get_request_body_string();

		return hash_hmac( 'sha512', $hashString, $this->store_key );
	}

	/**
	 * IFrame çağırma methodu.
	 *
	 * @return false|mixed
	 * @throws Exception
	 */
	public function frame_init() {
		$signature = $this->generate_signature( "v2/hepsipayframe/init" );

		return $this->post( "v2/hepsipayframe/init", $this->params( [
			'signature' => $signature,
		] ) );

	}

	/**
	 * Kullanıcının HepsiPay hesabının olup olmadığını sorgulayan servis.
	 *
	 * @return false|mixed
	 * @throws Exception
	 */
	public function check_availability() {
		$signature = $this->generate_signature( "v2/hepsipayframe/checkavailability" );

		return $this->post( "v2/hepsipayframe/checkavailability", $this->params( [
			'signature' => $signature,
		] ) );

	}

	/**
	 * Ödeme durumunu sorgulayan servis.
	 *
	 * @param   $token
	 *
	 * @return false|mixed
	 * @throws Exception
	 */
	public function payment_status( $token ) {

		$signature = $this->generate_signature( "payment/" . $token . "/status" );

		return $this->get( "payment/" . $token . "/status", $this->params( [
			'signature' => $signature,
		] ) );

	}
}
